import numpy as np
import pdb

def extract_mfcc(input, samplingRate=44100, frameRate=100, windowSize=256):
  """ ported over from Malcolm Slaney's Auditory Toolbox by Derek Tingle
  in a fevered, highly-caffeinated code sprint 03/2012 """
  #	Filter bank parameters
  lowestFrequency = 133.3333
  linearFilters = 13
  linearSpacing = 66.66666666
  logFilters = 27
  logSpacing = 1.0711703
  fftSize = 2*windowSize
  cepstralCoefficients = 13

  totalFilters = linearFilters + logFilters

  # monify! 
  # added by DT
  if len(np.shape(input))>1:
    input=monify(input)

  # Now figure the band edges.  Interesting frequencies are spaced
  # by linearSpacing for a while, then go logarithmic.  First figure
  # all the interesting frequencies.  Lower, center, and upper band
  # edges are all consequtive interesting frequencies. 

  freqs=np.zeros(totalFilters+2)
  freqs[0:linearFilters] = lowestFrequency + np.arange(0,linearFilters)*linearSpacing

  freqs[linearFilters:totalFilters+2] = freqs[linearFilters-1] * np.power(logSpacing,np.arange(1,logFilters+3))

  lower = freqs[0:totalFilters]
  center = freqs[1:totalFilters+1]
  upper = freqs[2:totalFilters+2]

  # We now want to combine FFT bins so that each filter has unit
  # weight, assuming a triangular weighting function.  First figure
  # out the height of the triangle, then we can figure out each 
  # frequencies contribution
  mfccFilterWeights = np.zeros((totalFilters,fftSize))
  triangleHeight = 2./(upper-lower)
  fftFreqs = np.arange(0,fftSize)/float(fftSize)*samplingRate

  for chan in range(totalFilters):
    mfccFilterWeights[chan,:] = \
        np.logical_and(fftFreqs>lower[chan],fftFreqs <=center[chan])* \
        triangleHeight[chan]*(fftFreqs-lower[chan])/(center[chan]-lower[chan])\
        + \
        np.logical_and(fftFreqs > center[chan],fftFreqs < upper[chan])* \
        triangleHeight[chan]*(upper[chan]-fftFreqs)/(upper[chan]-center[chan])

  #semilogx(fftFreqs,mfccFilterWeights')
  #axis([lower(1) upper(totalFilters) 0 max(max(mfccFilterWeights))])

  hamWindow=np.hamming(windowSize);

  # Figure out Discrete Cosine Transform.  We want a matrix
  # dct(i,j) which is totalFilters x cepstralCoefficients in size.
  # The i,j component is given by 
  #                cos( i * (j+0.5)/totalFilters pi )
  # where we have assumed that i and j start at 0.


  mfccDCTMatrix = 1/np.sqrt(totalFilters/2.)*np.cos(np.transpose(np.matrix(np.arange(0,cepstralCoefficients)))*np.matrix((2.*np.arange(0,totalFilters)+1))*np.pi/2./totalFilters)
  mfccDCTMatrix[0,:] = mfccDCTMatrix[0,:] * np.sqrt(2.)/2.

  #imagesc(mfccDCTMatrix)

  # Filter the input with the preemphasis filter.  Also figure how
  # many columns of data we will end up with.
  if False:
    preEmphasized = filter([1 -.97], 1, input)
  else:
    preEmphasized = input

  # modified by DT
  #windowStep = samplingRate/frameRate
  windowStep=windowSize/2

  cols = np.fix((len(input)-windowSize)/windowStep)
  if cols<1:
    return np.array(-1)

  # Allocate all the space we need for the output arrays.
  ceps = np.zeros((cepstralCoefficients, cols))

  #if (nargout > 1) freqresp = zeros(fftSize/2, cols) end
  #if (nargout > 2) fb = zeros(totalFilters, cols) end

  # Invert the filter bank center frequencies.  For each FFT bin
  # we want to know the exact position in the filter bank to find
  # the original frequency response.  The next block of code finds the
  # integer and fractional sampling positions.

  # if (nargout > 4)
  #   fr = (0:(fftSize/2-1))'/(fftSize/2)*samplingRate/2
  #   j = 1
  #   for i=1:(fftSize/2)
  #     if fr(i) > center(j+1)
  #       j = j + 1
  #     end
  #     if j > totalFilters-1
  #       j = totalFilters-1
  #     end
  #     fr(i) = min(totalFilters-.0001, ...
  #         max(1,j + (fr(i)-center(j))/(center(j+1)-center(j))))
  #   end
  #   fri = fix(fr)
  #   frac = fr - fri

  #   freqrecon = zeros(fftSize/2, cols)
  # end

  # Ok, now let's do the processing.  For each chunk of data:
  #    * Window the data with a hamming window,
  #    * Shift it into FFT order,
  #    * Find the magnitude of the fft,
  #    * Convert the fft data into filter bank outputs,
  #    * Find the log base 10,
  #    * Find the cosine transform to reduce dimensionality.
  for start in range(cols):
      first = start*windowStep 
      last = first + windowSize
      fftData = np.zeros((1,fftSize))
      fftData[0,0:windowSize] = input[first:last]*hamWindow
      fftMag = np.abs(np.fft.fft(fftData))
      earMag = np.log10(np.matrix(mfccFilterWeights)*np.transpose(np.matrix(fftMag)))
      ceps[:,start] = np.transpose(mfccDCTMatrix * earMag)

      #if (nargout > 1) freqresp(:,start+1) = fftMag(1:fftSize/2)' end
      #if (nargout > 2) fb(:,start+1) = earMag end
    #  if (nargout > 3) 
    #    fbrecon(:,start+1) = ...
    #      mfccDCTMatrix(1:cepstralCoefficients,:)' * ...
    #      ceps(:,start+1)
    #  end
    #if (nargout > 4)
    #  f10 = 10.^fbrecon(:,start+1)
    #  freqrecon(:,start+1) = samplingRate/fftSize * ...
    #    (f10(fri).*(1-frac) + f10(fri+1).*frac)
    #end

  # OK, just to check things, let's also reconstruct the original FB
  # output.  We do this by multiplying the cepstral data by the transpose
  # of the original DCT matrix.  This all works because we were careful to
  # scale the DCT matrix so it was orthonormal.

  #  if 1 & (nargout > 3) 
  #    fbrecon = mfccDCTMatrix(1:cepstralCoefficients,:)' * ceps

  #	imagesc(mt(:,1:cepstralCoefficients)*mfccDCTMatrix)

  return ceps

def extract_mfcc_and_rms(input, samplingRate=44100, frameRate=100, windowSize=256):
  """ ported over from Malcolm Slaney's Auditory Toolbox by Derek Tingle
  in a fevered, highly-caffeinated code sprint 03/2012 """
  #	Filter bank parameters
  lowestFrequency = 133.3333
  linearFilters = 13
  linearSpacing = 66.66666666
  logFilters = 27
  logSpacing = 1.0711703
  fftSize = 2*windowSize
  cepstralCoefficients = 13

  totalFilters = linearFilters + logFilters

  # monify! 
  # added by DT
  if len(np.shape(input))>1:
    input=monify(input)

  # Now figure the band edges.  Interesting frequencies are spaced
  # by linearSpacing for a while, then go logarithmic.  First figure
  # all the interesting frequencies.  Lower, center, and upper band
  # edges are all consequtive interesting frequencies. 

  freqs=np.zeros(totalFilters+2)
  freqs[0:linearFilters] = lowestFrequency + np.arange(0,linearFilters)*linearSpacing

  freqs[linearFilters:totalFilters+2] = freqs[linearFilters-1] * np.power(logSpacing,np.arange(1,logFilters+3))

  lower = freqs[0:totalFilters]
  center = freqs[1:totalFilters+1]
  upper = freqs[2:totalFilters+2]

  # We now want to combine FFT bins so that each filter has unit
  # weight, assuming a triangular weighting function.  First figure
  # out the height of the triangle, then we can figure out each 
  # frequencies contribution
  mfccFilterWeights = np.zeros((totalFilters,fftSize))
  triangleHeight = 2./(upper-lower)
  fftFreqs = np.arange(0,fftSize)/float(fftSize)*samplingRate

  for chan in range(totalFilters):
    mfccFilterWeights[chan,:] = \
        np.logical_and(fftFreqs>lower[chan],fftFreqs <=center[chan])* \
        triangleHeight[chan]*(fftFreqs-lower[chan])/(center[chan]-lower[chan])\
        + \
        np.logical_and(fftFreqs > center[chan],fftFreqs < upper[chan])* \
        triangleHeight[chan]*(upper[chan]-fftFreqs)/(upper[chan]-center[chan])

  #semilogx(fftFreqs,mfccFilterWeights')
  #axis([lower(1) upper(totalFilters) 0 max(max(mfccFilterWeights))])

  hamWindow=np.hamming(windowSize);

  # Figure out Discrete Cosine Transform.  We want a matrix
  # dct(i,j) which is totalFilters x cepstralCoefficients in size.
  # The i,j component is given by 
  #                cos( i * (j+0.5)/totalFilters pi )
  # where we have assumed that i and j start at 0.


  mfccDCTMatrix = 1/np.sqrt(totalFilters/2.)*np.cos(np.transpose(np.matrix(np.arange(0,cepstralCoefficients)))*np.matrix((2.*np.arange(0,totalFilters)+1))*np.pi/2./totalFilters)
  mfccDCTMatrix[0,:] = mfccDCTMatrix[0,:] * np.sqrt(2.)/2.

  #imagesc(mfccDCTMatrix)

  # Filter the input with the preemphasis filter.  Also figure how
  # many columns of data we will end up with.
  if False:
    preEmphasized = filter([1 -.97], 1, input)
  else:
    preEmphasized = input

  # modified by DT
  #windowStep = samplingRate/frameRate
  windowStep=windowSize/2

  cols = np.fix((len(input)-windowSize)/windowStep)
  if cols<1:
    return (np.array(-1), -1)

  # Allocate all the space we need for the output arrays.
  ceps = np.zeros((cepstralCoefficients, cols))


  # Ok, now let's do the processing.  For each chunk of data:
  #    * Window the data with a hamming window,
  #    * Shift it into FFT order,
  #    * Find the magnitude of the fft,
  #    * Convert the fft data into filter bank outputs,
  #    * Find the log base 10,
  #    * Find the cosine transform to reduce dimensionality.

  # store rms while we're already loopin thru it
  da_elm=0
  da_sum=0
  da_sqr=0.

  for start in range(cols):
      first = start*windowStep 
      last = first + windowSize
      fftData = np.zeros((1,fftSize))
      cur_input=input[first:last]
      #keepin track of rms
      da_elm+=windowSize
      da_sum+=np.sum(cur_input)
      da_sqr+=np.sum(np.power(cur_input,2))
      fftData[0,0:windowSize] = cur_input*hamWindow
      fftMag = np.abs(np.fft.fft(fftData))
      earMag = np.log10(np.matrix(mfccFilterWeights)*np.transpose(np.matrix(fftMag)))
      ceps[:,start] = np.transpose(mfccDCTMatrix * earMag)

  da_rms=np.sqrt(da_sqr/da_elm)
  return (ceps,da_rms)

def monify(sig):
  """ if signal appears to be stereo, avg the two channels and return"""
  if len(np.shape(sig))>1:
    return np.mean(sig,1)
  else:
    return sig

def extract_rms(sig):
  #return np.sqrt(np.mean(np.power(monify(sig),2)))
  return np.sqrt(np.mean(np.power(sig,2)))

if __name__=='__main__':
  y=np.array(map(float,open('battles.csv','r').read().rstrip().split('\n')))
  ceps=mfcc(y,windowSize=4096)
  pdb.set_trace()
