from brick.tour.models import *
from django.conf import settings
from django.db.models import Avg,StdDev
from struct import unpack
from audio_featzor import *
from time import time

import pdb

def extract_audio_feats(uuid,populate_db=False,windowSize=4096):
  if populate_db:
    (audi,created)=Audio.objects.get_or_create(uuid=uuid)
    if not created:
      af=AudioFeats.objects.filter(audio=audi, win_size=windowSize)
      if af:
        return af[0]

  audio_fid=open(settings.MEDIA_ROOT + uuid,'rb')
  bin_sig= audio_fid.read()

  sig=[]

  for i in range(len(bin_sig)/8):
    x= unpack('ff',bin_sig[i*8:(i+1)*8])
    # monify... should be identical in both channels
    sig.append(x)

  parm_dict={}
  if populate_db:
    parm_dict['audio']=audi
  parm_dict['win_size']=windowSize

  (ceps, da_rms)=extract_mfcc_and_rms(sig,windowSize=windowSize)
  if ceps.size==1:
    return

  parm_dict['rms']=da_rms


  m_ceps=np.mean(ceps,1)
  v_ceps=np.var(ceps,1)

  for i in range(len(m_ceps)):
    parm_s='mfcc' + str(i+1)
    parm_dict[parm_s + '_mean']=m_ceps[i]
    parm_dict[parm_s + '_var']=v_ceps[i]

  if populate_db:
    af=AudioFeats(**parm_dict)
    af.save()

  return af

def extract_all_feats(windowSize=4096):
  audis=Media.objects.filter(media_type=0)
  for audi in audis:
    t0=time()
    #extract_audio_feats(audi.uuid,True,windowSize=8192)
    t1=time()
    extract_audio_feats(audi.uuid,True,windowSize=windowSize)
    #t2=time()
    #print "%0.2f\t%0.2f"%(t1-t0,t2-t1)
    #extract_audio_feats(audi.uuid,False)
    #print time()-t0

def get_stats():
  stats=AudioFeats.objects.aggregate(
      Avg('rms'),
      Avg('mfcc1_mean'),
      Avg('mfcc2_mean'),
      Avg('mfcc3_mean'),
      Avg('mfcc4_mean'),
      Avg('mfcc5_mean'),
      Avg('mfcc6_mean'),
      Avg('mfcc7_mean'),
      Avg('mfcc8_mean'),
      Avg('mfcc9_mean'),
      Avg('mfcc10_mean'),
      Avg('mfcc11_mean'),
      Avg('mfcc12_mean'),
      Avg('mfcc13_mean'),
      StdDev('rms'),
      StdDev('mfcc1_mean'),
      StdDev('mfcc2_mean'),
      StdDev('mfcc3_mean'),
      StdDev('mfcc4_mean'),
      StdDev('mfcc5_mean'),
      StdDev('mfcc6_mean'),
      StdDev('mfcc7_mean'),
      StdDev('mfcc8_mean'),
      StdDev('mfcc9_mean'),
      StdDev('mfcc10_mean'),
      StdDev('mfcc11_mean'),
      StdDev('mfcc12_mean'),
      StdDev('mfcc13_mean'))

  return stats;

def norm_af_vect(af,stats):
  """
  takes an AudioFeature object and a stats dictionary and returns a vector
  of normalized audio features.
  normalized means mean removed divided by stdev
  """
  # assumes 14 dims
  vectR=np.zeros(14)
  vectR[0]=(af.rms-stats['rms__avg'])/stats['rms__stddev']

  afd=af.__dict__
  for i in range(1,14):
    parm_s='mfcc' + str(i) + '_mean'
    vectR[i]=(afd[parm_s]-stats[parm_s + '__avg'])/stats[parm_s + '__stddev']

  return vectR


def find_closest_audio(audi_uuid):
  af=extract_audio_feats(audi_uuid,True)
  stats=get_stats()

  stats['mfcc1_mean__stddev']*=.3
  stats['mfcc2_mean__stddev']*=.3
  stats['mfcc3_mean__stddev']*=.3
  stats['mfcc4_mean__stddev']*=.3
  stats['mfcc5_mean__stddev']*=.3

  cands=[]
  while len(cands)<30:
    cands=AudioFeats.objects.filter(
        mfcc1_mean__gt=af.mfcc1_mean-np.abs(stats['mfcc1_mean__stddev']),
        mfcc1_mean__lt=af.mfcc1_mean+np.abs(stats['mfcc1_mean__stddev']),
        mfcc2_mean__gt=af.mfcc2_mean-np.abs(stats['mfcc2_mean__stddev']),
        mfcc2_mean__lt=af.mfcc2_mean+np.abs(stats['mfcc2_mean__stddev']),
        mfcc3_mean__gt=af.mfcc3_mean-np.abs(stats['mfcc3_mean__stddev']),
        mfcc3_mean__lt=af.mfcc3_mean+np.abs(stats['mfcc3_mean__stddev']),
        mfcc4_mean__gt=af.mfcc4_mean-np.abs(stats['mfcc4_mean__stddev']),
        mfcc4_mean__lt=af.mfcc4_mean+np.abs(stats['mfcc4_mean__stddev']),
        mfcc5_mean__gt=af.mfcc4_mean-np.abs(stats['mfcc5_mean__stddev']),
        mfcc5_mean__lt=af.mfcc4_mean+np.abs(stats['mfcc5_mean__stddev']))

    stats['mfcc1_mean__stddev']*=1.25
    stats['mfcc2_mean__stddev']*=1.25
    stats['mfcc3_mean__stddev']*=1.25
    stats['mfcc4_mean__stddev']*=1.25
    stats['mfcc5_mean__stddev']*=1.25

  #ya a little wasteful... whatever
  stats=get_stats()
  afv=norm_af_vect(af,stats)
  dstlst=[]

  for c in cands:
    cfv=norm_af_vect(c,stats)
    dst=np.linalg.norm(afv-cfv)
    dstlst.append((dst,c.audio))

  dstlst.sort()

  return dstlst[:15]

if __name__=='__main__':
  #a=Audio.objects.all().order_by('?')[0]

  t=time()
  find_closest_audio(a.uuid)
  print time()-t
  #extract_all_feats(windowSize=8192)


