from brick.tour.models import *
from django.conf import settings
from django.db.models import Avg,StdDev
from struct import unpack
from pylab import imread,imshow,show,figure
from time import sleep,time
import numpy as np
import pdb

NUM_ROWS=4
NUM_COLS=3

def extract_hist(image_uuid,populate_db=False,num_rows=NUM_ROWS,num_cols=NUM_COLS):
  if populate_db:
    #TODO make this the key... remove this table?
    (img,created) = Image.objects.get_or_create(uuid=image_uuid)

  img_data=imread(settings.MEDIA_ROOT + image_uuid)

  # images are rotated, so the first dimension acutally corresponds to columns
  (pix_cols, pix_rows,NUM_COLORS)=np.shape(img_data)

  col_width = pix_cols/num_cols # width of each column in pixels
  row_height = pix_rows/num_rows# height of each row in pixels

  color_objs=Color.objects.all()

  hist=np.zeros((num_rows,num_cols,NUM_COLORS))
  for i in range(num_rows):
    for j in range(num_cols):
      for c in range(NUM_COLORS): #each color
        val=np.mean(img_data[j*col_width:(j+1)*col_width,i*row_height:(i+1)*row_height,c])/256.

        hist[i,j,c]=val

      if populate_db:
        this_block=ImageHistBlock(
            image=img,
            grid_row=i,
            grid_col=j,
            num_rows=num_rows,
            num_cols=num_cols,
            red=hist[i,j,0],
            green=hist[i,j,1],
            blue=hist[i,j,2])
        this_block.save()

  return hist

def pull_hist(img,num_rows=NUM_ROWS,num_cols=NUM_COLS):
  NUM_COLORS=3
  hist=np.zeros((num_rows,num_cols,NUM_COLORS))

  blocks=ImageHistBlock.objects.filter(image=img,num_cols=num_cols,num_rows=num_rows)
  for b in blocks:
    hist[b.grid_row,b.grid_col,0]=b.red
    hist[b.grid_row,b.grid_col,1]=b.green
    hist[b.grid_row,b.grid_col,2]=b.blue

  return hist

def show_img(img):
  figure()
  img_data=imread(settings.MEDIA_ROOT + img.uuid)
  imshow(img_data)

def find_closest_hist1(image_uuid,num_rows=NUM_ROWS,num_cols=NUM_COLS):
  hist=extract_hist(image_uuid,num_rows=num_rows,num_cols=num_cols)
  hist2=extract_hist(image_uuid,num_rows=1,num_cols=1)
  
  imgs=Image.objects.all()

  dstlst=[]
  for img in imgs:
    dst=np.linalg.norm(pull_hist(img,num_rows,num_cols)-hist)
    dst2=np.linalg.norm(pull_hist(img,1,1)-hist2)
    dstlst.append((dst,dst2,img))
  dstlst.sort()

  return dstlst

def find_closest_hist2(image_uuid):
  hist=extract_hist(image_uuid,num_rows=1,num_cols=1)

  imgs=Image.objects.all()
  dstlst=[]
  t0=time()
  for img in imgs:
    dst=np.linalg.norm(pull_hist(img,num_cols=1,num_rows=1)-hist)
    dstlst.append((dst,img.id))
  dstlst.sort()
  img_keeps=[id for (d,id) in dstlst[:len(dstlst)/10]]
  print time()-t0

  hist=extract_hist(image_uuid)

  dstlst=[]
  for img in Image.objects.filter(id__in=img_keeps):
    dst=np.linalg.norm(pull_hist(img)-hist)
    dstlst.append((dst,img))
  dstlst.sort()

  return dstlst

def find_closest_hist3(image_uuid,num_close=30):
  hist=extract_hist(image_uuid,num_rows=1,num_cols=1)
  thresh=np.ceil(Image.objects.count()/5.)

  stats=ImageHistBlock.objects.filter(num_rows=1,num_cols=1).aggregate(
      Avg('red'),
      Avg('green'),
      Avg('blue'),
      StdDev('red'),
      StdDev('green'),
      StdDev('blue'))
  
  stats['red__stddev']*=.3
  stats['green__stddev']*=.3
  stats['blue__stddev']*=.3

#  cands=[]
#  while len(cands)<thresh:
#    cands=ImageHistBlock.objects.filter(
#        num_cols=1,
#        num_rows=1,
#        red__gt=hist[0,0,0]-stats['red__stddev'],
#        red__lt=hist[0,0,0]+stats['red__stddev'],
#        green__gt=hist[0,0,0]-stats['green__stddev'],
#        green__lt=hist[0,0,0]+stats['green__stddev'],
#        blue__gt=hist[0,0,2]-stats['blue__stddev'],
#        blue__lt=hist[0,0,2]+stats['blue__stddev'])
#
#    stats['red__stddev']*=1.25
#    stats['green__stddev']*=1.25
#    stats['blue__stddev']*=1.25
  
  cands=ImageHistBlock.objects.all()
  img_keeps=[histblock.image_id for histblock in cands]

  hist=extract_hist(image_uuid)

  dstlst=[]
  for img in Image.objects.filter(id__in=img_keeps):
    dst=np.linalg.norm(pull_hist(img)-hist)
    dstlst.append((dst,img))
  dstlst.sort()

  iuuids=[i.uuid for (dist,i) in dstlst[:num_close]]
  mps=[]

  for iuuid in iuuids:
    res=MediaPair.objects.filter(image_uuid=iuuid)
    if res:
      mps.append(res[0])
    else:
      'missing an iuuid somewhere...'

  return mps

def extract_all_feats(num_rows=NUM_ROWS,num_cols=NUM_COLS):
  imgs=Media.objects.filter(media_type=1)
  for img in imgs:
    extract_hist(img.uuid,True,num_rows,num_cols)

  return

def __main__():
  #extract_all_feats(1,1)

#  t0=time()
#  dstlst=find_closest_hist1('A10343D0-8DB6-41E8-899C-EE4B242BBD65')
#  print "method 1 took " + str(time()-t0) + " seconds to parse " + str(len(dstlst)) + " files"

  #rr=Image.objects.all().order_by('?')[0]
  rr=Image.objects.get(uuid='490ED2DA-AAD8-4D62-BC24-51D494053D84')

  t0=time()
  mps=find_closest_hist3(rr.uuid)
  print "method 3 took " + str(time()-t0) + " seconds to parse " + str(len(mps)) + " files"

  t0=time()
  dstlst=find_closest_hist2(rr.uuid)
  print "method 2 took " + str(time()-t0) + " seconds to parse " + str(len(dstlst)) + " files"

  pdb.set_trace()

if __name__=='__main__':
  __main__()


#tmp[j*col_width:(j+1)*col_width,i*row_height:(i+1)*row_height,c]=val

#figure(1)
#imshow(tmp)
#
#figure(0)
#imshow(img_data)
#show()

